#    most of this task is now taken from the standard application

set VERSION "MIDAS"

#    put some default environment variables before loading common 

if {[info exists env(RDOUThdwre)] == 0} then {set env(RDOUThdwre) VME} 

loadsource sys.common

set frametitle "MIDAS Data Acquisition Experiment Control"

set VMEPresent 1
set VMEEnable 1

set TSEnable 1

set histEnable 1
set HistMode 1

set txEnable 1

set TestBuffer ""

lappend actionlist [list "Show histogram name translations" "Show_Xlate"]
lappend actionlist [list "Edit histogram translation file"  "edit_xlate"]
lappend actionlist [list "Raw Data Display"  "Dump_Raw_VME_Data"]


proc create_spectra {} {
   global SpectrumServer HistMode EGreport Options

   if {$HistMode == 0} {return}

   set-footer "Creating spectra"

   set EGreport_SAVE $EGreport
   set EGreport -1
   
   Make_Xlate $SpectrumServer

   set range $Options(Stat.channels)
   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Stat' !new)" } m]
   map_spectrum "Stat"

   set range $Options(Rate.channels)
   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Rate' !new)" } m]

#   for silena 9418

    create_s9418_spectra
   
#    for CAEN V767

    create_v767_spectra

#    for CAEN V890

    create_v890_spectra
   
   set EGreport $EGreport_SAVE

   clear-footer
}

proc define_VME_addresses {} {

#    load hardware configuration into readout software

     ClearConfig

#    the S9418 VME modules

     define_s9418_config

#    the V767 VME modules

     define_v767_config

#    the V890 VME modules

     define_v890_config

#    then the acquisition control module

     define_sac_config
}

proc click-vme {action} {

   starttask vmecontrol

    switch $action {
      0    {set p DAQ-halt}
      1    {set p DAQ-go}
      2    {set p DAQ-reset}
      3    {set p DAQ-setup}
    }

   set-busy
 
   set-footer "sending $p to VME control task"
   set z [catch {send vmecontrol $p} m]

   clear-busy
   if {$m == ""} then {clear-footer} else {set-footer $m}

   if {$action == 3} then {
       define_VME_addresses
       do_v767_setup
       do_v890_setup
   }

   redisplay-vme
}

proc redisplay-vme {} {
    global VMEEnable vmestate

  if {$VMEEnable == 0} then {return}

   set vmestate [expr [read-register sac#1.Enable] & 1]
   :modify VME :value $vmestate
}

proc Stop_Electronics {} {

       write-register sac#1.STR 0
}

proc Go_Electronics {} {

    write-register sac#1.MON0 2
    write-register sac#1.STR 9
    write-register sac#1.ARM 0
}

proc Reset_Electronics {} {write-register sac#1.EVCLR 0}

proc Dump_Raw_VME_Data {} {

   set info ""

   set maxmodule [inform vmecontrol inquire s9418count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   for {set i 1} {$i <= $maxmodule} {incr i} {
      append info "\nS9418#$i\n"
      execute-command 19 "Getting Raw Data for S9418#$i" "2 $i" 2
      set data [read-register EXEC.Response]

     foreach word $data {
         set word 0x$word
         if {$word == 0x55aaaa55} then {
             append info "$word - end of event\n"
             continue
         }
         if {$word == 0xaa5555aa} then {
             append info "$word - end of dump\n"
             continue
         }
         set type [expr $word >> 30 & 3]
         switch $type {
           0 {
             set event_type "data"
             set channel [expr $word >> 20 & 0x1f]
             set adc [expr $word  & 0xfff]
             append info "[format "0x%8x    %8s   channel= %5i;  adc= 0x%3x" $word $event_type $channel $adc]"
           }
           1 {
             set event_type "footer"
             set VSN [expr $word & 0xffff]
             set WC [expr $word >> 16 & 0x3f]
             set EvCount [expr $word >> 25 & 0xf]
             append info "[format "0x%8x    %8s   vsn= 0x%4x;   wc= %3i; Ev Counter= %3i" $word $event_type $VSN $WC $EvCount]"
           }
           2 -
           3 {
             set event_type "end event"
             append info "[format "0x%8x    %8s" $word $event_type]"
           }
         }
         append info "\n"
     }
     
   }

   set maxmodule [inform vmecontrol inquire v767count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   for {set i 1} {$i <= $maxmodule} {incr i} {
      append info "\nV767#$i\n"
      execute-command 19 "Getting Raw Data for V767#$i" "1 $i" 2
      set data [read-register EXEC.Response]
      
     foreach word $data {
        set word 0x$word
         if {$word == 0x55aaaa55} then {
             append info "$word - end of event\n"
             continue
         }
         if {$word == 0xaa5555aa} then {
             append info "$word - end of dump\n"
             continue
         }
         set type [expr $word >> 21 & 3]
         switch $type {
            0 {
             set event_type "data"
             set channel [expr $word >> 24 & 0x7f]
             set st [expr $word >> 23 & 1]
             set ed [expr $word >> 20 & 1]
             set tdc [expr $word  & 0xfffff]
             append info "[format "0x%8x    %8s   channel= %5i; start= %3i; edge= %3i; time= 0x%5x" $word $event_type $channel $st $ed $tdc]"
            }
            2 {
             set event_type "header"
             set geo [expr $word >> 27 & 0x1f]
             set counter [expr $word & 0x3ff]
             append info "[format "0x%8x    %8s   module= %5i; event number= %6i" $word $event_type $geo $counter]"
            }
            1 {
             set event_type "trailer"
             set geo [expr $word >> 27 & 0x1f]
             set status [expr $word >> 24 & 7]
             set counter [expr $word & 0xffff]
             append info "[format "0x%8x    %8s   module= %5i; status=%2i; event data counter= %8i" $word $event_type $geo $status $counter]"
            }
            3 {
             append info "$word - assumed end of data\n"
             midas-information $info
             clear-busy
             set EGreport $EGreport_Save
             return
            }
         }
         append info "\n"
     }
   }

   midas-information "$info"
}


proc midas-information {text} {

    set frame [self frame-name].info

    :frame $frame :label  "Information from [self frame-title]" :toplevel T
         wm protocol $frame WM_DELETE_WINDOW "destroy $frame"

    :galley G :deaf T :strings $text :font fixed
    :show T
}


